/*****************************************************************************/
/* Transphorm totem-pole PFC on 28335                                        */
/* November 2015                                                             */
/* Modify to 1.2kW/2.4kW Power Rating @115Vac/230Vac                         */
/* Fsw = 100kHz                                                              */
/* Version 1.2                                                               */
/* Modify current controller for Low Line and High Line						 */
/* Solve bugs                                                                */
/*****************************************************************************/

#include "DSP28x_Project.h"
#include "math.h"

// Functions that will be run from RAM need to be assigned to
// a different section.  This section will then be mapped using
// the linker cmd file.
#pragma CODE_SECTION(epwm1_isr, "ramfuncs");

// Marco for limiting calculated output
#define	UpDownLimit16(Value,UpValue,DownValue)		  		  \
		{											  		  \
			Value = (Value >= UpValue) ? UpValue : Value;	  \
			Value = (Value <= DownValue) ? DownValue : Value; \
		}
#define	UpDownLimitfloat(Value,UpValue,DownValue)	  		  \
		{											  		  \
			Value = (Value >= UpValue) ? UpValue : Value;	  \
			Value = (Value <= DownValue) ? DownValue : Value; \
		}


// Prototype statements for functions found within this file.
void InitEPwm1Gpio(void);
void InitEPwm2Gpio(void);
void InitEPwm3Gpio(void);

void InitEPwm1Example(void);
void InitEPwm2Example(void);
void InitEPwm3Example(void);

//config ADC at the beginning to sample Vref
void ConfigAdc1(void);
//config ADC to sample current and voltage sequentially for control purpose
void ConfigAdc2(void);
void InitGPIOs(void);

interrupt void epwm1_isr(void);
interrupt void epwm3_isr(void);

// These are defined by the linker
extern Uint16 RamfuncsLoadStart;
extern Uint16 RamfuncsLoadEnd;
extern Uint16 RamfuncsRunStart;

// Global variables
Uint16  Int1Count;
Uint16  EPwm3TimerIntCount;

Uint16 Sum;
Uint16 IL;
Uint16 I_zero;
Uint16 Udc_in1;
Uint16 Udc_in2;
int16 Udc_in;
int16 Udc;
Uint16 Uac_p;
Uint16 Uac_n;
Uint16 Latch_counter;
Uint16 Udcref;
Uint16 Udcref1;
int16 Uerr;
int16 Uerr_last;
int16 Icntr;


float UregPout;

float UregIout;

float UregPIout;
float UregPIout1;
float UregIout_pre;
float UregPIout_pre;

float KpV;
float KiV;
Uint16 Vrect;
int16 Irect;
float VSqrtSum;

Uint16 MainPoint;

long Vrms;
int16 Vrms_f;
Uint16 VLine_flag; //high line = 1; low line = 0;

float VregILimit;
float VregPILimitUpper;
float VregPILimitLower;
float VregPILimitBurst;

int16 Ierr;
int16 Ierr_1;
int16 Duty;
int16 Duty_1;
float inv_Vrms;
float inv_Udc;
int16 Duty_temp;
int16 Duty_out;
Uint16 Vth;
Uint16 Vth2;

float K1I;
float K2I;
float UPI_Iref_out;

Uint16 sscounter;
Uint16 lightload;

float tempf1;
float tempf2;
float tempf3;
Uint16 tempu1;
int16 tempi1;


// System Flags
struct sFlag
{
	Uint16 Pol 				:1; // bit 0:  Pol=1, Positive cycle, Pol=0, Negative cycle
	Uint16 Pol_last 		:1;	// bit 1: 1: positive cycle, 0: negative cycle
	Uint16 Latch 			:1; // bit 2: Latch flag 0 effective, when Latch = 0, do not read the polarity
	Uint16 Stop 			:1; // bit 3, 1: stop the PWM  0: enable the PWM.
	Uint16 Fault 			:1; // bit 4, 1: fault condition 0: no fault condition
	Uint16 ShutDown 		:1; // Flag to shutdown, turn-off all gate signals.
	Uint16 Latch_last 		:1; // latch signal of 100uS ago, for used in the 10KHz slow loop.
	Uint16 EnableSync 		:1; // enable synchronous driving the coolmos
	Uint16 HalfCharged 		:1; // at the beginning of softstart, this bit show if the bulk cap is charged or not 1: half-charged 0: not charged
	Uint16 BurstTurnOff 	:1; // burst mode off bit, 1: turn-off all gate temperoraily; 0: turn-on as in normal operation
	Uint16 SoftStartOver	:1; // flag for soft-start over; 1: soft-start over, converter in steady-states 0: converter is in soft-start, do not check for burst operation
	Uint16 BurstMode 		:1; // flag for burst mode status, 1: in burst mode.  When in burst mode, current reference is fixed when burst.
	Uint16 LightLoad 		:1;	// flat for Light Load condition
	Uint16 LightLoad_last 	:1; // flag for Light Load condition last control cycle
	Uint16 Rsvd 			:2;	// reserved 2 bits
}sysFlag;


void main(void)
{
	// Initialize System Control:
	// PLL, WatchDog, enable Peripheral Clocks
	// This function function is found in the DSP2833x_SysCtrl.c file.
    InitSysCtrl();

    EALLOW;
    SysCtrlRegs.HISPCP.all = 0x3;	// HSPCLK = SYSCLKOUT/ADC_MODCLK=25MHz
    EDIS;


	// Initalize GPIO:
	// init GPIO pins for ePWM1, ePWM2, ePWM3
	// These functions are at the end of this file
	InitEPwm1Gpio();
	InitEPwm2Gpio();
	InitEPwm3Gpio();

	InitGPIOs();

	// Clear all interrupts and initialize PIE vector table:
	// Disable CPU interrupts
	DINT;

	// Initialize the PIE control registers to their default state.
	// The default state is all PIE interrupts disabled and flags
	// are cleared.
	// This function is found in the DSP2833x_PieCtrl.c file.
	InitPieCtrl();

	// Disable CPU interrupts and clear all CPU interrupt flags:
	IER = 0x0000;
	IFR = 0x0000;

	// Initialize the PIE vector table with pointers to the shell Interrupt
	// Service Routines (ISR).
	// This will populate the entire table, even if the interrupt
	// is not used
	// The shell ISR routines are found in DSP2833x_DefaultIsr.c.
	// This function is found in DSP2833x_PieVect.c.
	InitPieVectTable();

	// Interrupts that are used in this example are re-mapped to
	// ISR functions found within this file.
	EALLOW;
	PieVectTable.EPWM1_INT = &epwm1_isr;
	PieVectTable.EPWM3_INT = &epwm3_isr;
	EDIS;

	// Copy time critical code and Flash setup code to RAM
	// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
	// symbols are created by the linker. Refer to the F28335.cmd file.
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

	// Call Flash Initialization to setup flash wait states
	// This function must reside in RAM
	InitFlash();

	InitAdc();


	// initialize variables
	Sum = 0;

	EPwm3TimerIntCount = 0;
	Latch_counter = 0;

	// 195*4.63873-10=895
	Udcref = 895;
	Udcref1 = 895;
	Uerr = 0;
	Uerr_last = 0;
	Icntr = 0;
	Udc_in = 0;
	Udc = 0;
	Vth = 20;
	Vth2 = 10;  //Vth2 for current control

	UregPout = 0.0;
	// initially will give out a very small current 0.4*175.88*55*9.2398= 35752
	UregIout = 35752.0;
	UregPIout = 0.0;
	UregPIout1 = 0.0;
	// for 55Vac,  0.5A*175.83/(55*9.2398)=0.1772 unecessary, just UregIout is enough
	UPI_Iref_out = 0.0;
	Vrms_f = 10;
	Vrms = 10;

	VLine_flag = 0; //low line default

	KpV = 1334.456;    //230Vin
	KiV = 0.314027; //230Vin
	//KpV = 2140;    //115Vin
	//KiV = 0.503655; //115Vin


	Ierr = 0;
	Ierr_1 = 0;
	Duty = 10;
	Duty_1 = 10;

	//initial parameters are for low line;
	K1I = 0.347452;             //   115Vin
	K2I = 0.326258;       		//   115Vin


	sysFlag.Stop = 1;  			// at the beginning stop the PWM.
	sysFlag.Fault = 0;  		// not fault at the beginning
	sysFlag.Latch = 1;  		// disable latch at the beginning
	sysFlag.Pol = 1; 			// initial value 1
	sysFlag.Pol_last = 1; 		// initial value 1
	sysFlag.Latch_last = 1; 	// disable latch at the beginning
	sysFlag.EnableSync = 0;
	sysFlag.HalfCharged = 0;  	 // initial value 0, this bit is changed to 1 when DC bus reached an level
	sysFlag.BurstTurnOff = 0;    // initial value 0, no burst turn-off
	sysFlag.BurstMode = 0;   	 // initial value 0, no burst mode, do not turn off gate momentarily
	sysFlag.ShutDown = 0;   	 // shutdown bit initial value 0, not shutting down
	sysFlag.SoftStartOver = 0;   // initial value 0, converter has not completed soft-start sequence
	sysFlag.LightLoad = 0;       // no light load
	sysFlag.LightLoad_last = 0;
	Int1Count = 0;
	sscounter = 0;

	lightload = 0;     // check light load condition counter

	// configure ADC to sample Vref get the Vref value
	ConfigAdc1();
	asm(" RPT #10 || NOP");

	// immediate reset Sequencer to CONV00 and wait for trigger
	AdcRegs.ADCTRL2.bit.RST_SEQ1 = 1;
	// Software trigger
	AdcRegs.ADCTRL2.bit.SOC_SEQ1 = 1;

   	// must wait at least 30 cycles for the BSY bit to show up
   	asm(" RPT #100 || NOP");
   	asm(" RPT #100 || NOP");
   	asm(" RPT #100 || NOP");

    for (	tempi1 = 0; \
    		((tempi1 < 100) && (AdcRegs.ADCST.bit.SEQ1_BSY == 1)); \
    		tempi1 ++ )
  	{
  		asm(" NOP");
  	}

	Sum += AdcRegs.ADCRESULT0 >> 4;
	Sum += AdcRegs.ADCRESULT1 >> 4;
	Sum += AdcRegs.ADCRESULT2 >> 4;
	Sum += AdcRegs.ADCRESULT3 >> 4;
	I_zero = Sum >> 2;

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
	EDIS;

	InitEPwm1Example();
	InitEPwm2Example();
	InitEPwm3Example();

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
	EDIS;

	//Configure ADC for control purpose, sample current and voltage
	ConfigAdc2();

	// must wait at least 30 cycles for the BSY bit to show up
	asm(" RPT #100 || NOP");
	asm(" RPT #100 || NOP");
	asm(" RPT #100 || NOP");

	// Enable interrupts
	// Enable CPU INT3 which is connected to EPWM1(100KHz) and EPWM3(10KHz) INT:
	IER |= M_INT3;

	// Enable EPWM INTn in the PIE: Group 3 interrupt 1-3
	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;
	PieCtrlRegs.PIEIER3.bit.INTx3 = 1;

	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM


	for(;;)
	{
		asm("  NOP");
		asm("  NOP");
	}

}

interrupt void epwm1_isr(void)
{
	// Determine the polarity while waiting for ADC to finish.
	// Determine the polarity only when Latch is 1 disabled.
	if(sysFlag.Latch)
	{
		sysFlag.Pol = GpioDataRegs.GPADAT.bit.GPIO17;
	}

	// Polarity changes, which means a zero-crossing is detected!
	if(sysFlag.Pol != sysFlag.Pol_last)
	{
		// Enable latch, will not read polarity again for the next 300 cycles
		sysFlag.Latch = 0;
		Latch_counter = 0;
	}

	// Enable latch for 3 ms
	if(Latch_counter > 300)
	{
		//latch is cleared, will read polarity again.
		sysFlag.Latch = 1;
		// Latch counter does not increase anymore when latch disabled.
		Latch_counter = 300;
	}

	// if fault or stop or BurstTurnOff, shutdown all PWM gates
	sysFlag.ShutDown = sysFlag.Fault || sysFlag.Stop || sysFlag.BurstTurnOff;

	// before the algorithm or update PWM cmpr register, check flag status and do trip event accordingly
	// if there is a fault condition or stop by software, or Burst modeOFF, turn off all gates temporarily
	if(sysFlag.ShutDown == 1)
	{
		EALLOW;
		EPwm1Regs.TZFRC.bit.OST = 1;
		EPwm2Regs.TZFRC.bit.OST = 1;    // one short trip for both channel
		EDIS;
		GpioDataRegs.GPACLEAR.bit.GPIO6 = 1;   //set output low for high side coolMos
		GpioDataRegs.GPACLEAR.bit.GPIO7 = 1;   // set output low for low side coolMos
	}
	else
	{
		EALLOW;
		EPwm1Regs.TZCLR.bit.OST = 1;
		EPwm2Regs.TZCLR.bit.OST = 1;
		EDIS;

	}

    for (	tempi1 = 0;	\
    		((tempi1 < 100) && (AdcRegs.ADCST.bit.SEQ1_BSY == 1)); \
    		tempi1 ++)
  	{
  		asm(" NOP");
  	}

	// read all ADC registers, IL, Udc, Uac_p, Uac_n
    // IL: Inductor instataneous current value
    // Udc_in: DC bus voltage value
    // Uac_p: 1.5V Reference Value
    // Uac_n: AC input neutral line voltage value

	IL 			= AdcRegs.ADCRESULT0 >> 4;
	Udc_in 		= AdcRegs.ADCRESULT1 >> 4;
	Uac_p 		= AdcRegs.ADCRESULT2 >> 4;
	Uac_n 		= AdcRegs.ADCRESULT3 >> 4;
	//T_heatsink 	= AdcRegs.ADCRESULT4 >> 4;

	// reset ADC sequencer, wait for the next SOC from PWM1
	// immediate reset Sequencer to CONV00 and wait for trigger
	AdcRegs.ADCTRL2.bit.RST_SEQ1 = 1;

	// Calculate the rectified voltage, this rectified voltage will be used to calculate RMS
	// Absolute value of input voltage
	if (Uac_p > Uac_n)
	{
		Vrect = Uac_p - Uac_n;
	}
	if (Uac_p <= Uac_n)
	{
		Vrect = Uac_n - Uac_p;
	}

	// Calculate the rectified current
	// In positive half, feedback IL-I_zero;
	if (sysFlag.Pol == 1)
	{
		Irect = IL - Uac_p;
	}
	//In negative half, feedback I_zero-IL
	if (sysFlag.Pol == 0)
	{
		Irect = Uac_p - IL;
	}

	//digital filter on Udc
	Udc += ((Udc_in - Udc) >> 2);

	// Do control and everything when counter is not 0. When is 0, reconfigure PWM
	if(Latch_counter != 0)
	{
		//start the current and voltage loop.

		//if there is no shutdown, and we have started the control and loop.
		if(sysFlag.ShutDown == 0)
		{
			Icntr = (int)(UPI_Iref_out * Vrect);

			// Inner current loop, output duty ratio D
			Ierr = Icntr - Irect;

			//open-loop, do not control when input voltage is very small.
			// high line 20, low line changed to 10??
			if(Vrect < Vth2)
			{
				// No regulation when Vin is too low.
				// Current error can not be trusted.
				Ierr = 0;
			}

			tempf1 = Ierr * K1I;
			tempf2 = Ierr_1 * K2I;
			tempf1 = tempf1 - tempf2;
			Duty = Duty_1 + (int)tempf1;

			//added input voltage feed forward.
			tempi1 = (int)(Vrect * inv_Udc);

			// limit the Vrec/Vdc to be less than 0.9
			tempi1 = ( tempi1 >= 675)? 675 : tempi1;

			// Duty output = 750 - Duty_temp + Duty(current controller)
			//750-Duty_temp is feedforward part.
			Duty_out = 750 - tempi1 + Duty;

			// high line 20, low line changed to 10?
			if(Vrect < Vth2)
			{
				// no regulation Vrect<20, when Vrect >20, the Duty_last is 749 to start with
				Duty_out = 749;
			}
			UpDownLimit16(Duty_out, 749, 0);

			//update the CMPR register according to the polarity
			// when polarity just changes, the first few cycles needs soft-start
			// the polarity reading must be latched in the first few cycles.

			//if in negative half, update cmpr accordingly
			if(sysFlag.Pol == 0)
			{
				// Soft-start for the first 10 cycles
				if(Latch_counter < 10)
				{
					// HS GaN duty increases gradually
					// counter multiply by 80
					EPwm1Regs.CMPA.half.CMPA = ((Latch_counter << 4) + (Latch_counter << 6));

					// LS GaN keeps 0% duty
					EPwm2Regs.CMPA.half.CMPA = 750;
				}
				else
				{
					//HS GaN is the active switch, duty = Duty_out
					EPwm1Regs.CMPA.half.CMPA = Duty_out;

					//LS GaN is SR switch, duty = 1- Duty_out
					EPwm2Regs.CMPA.half.CMPA = EPwm1Regs.CMPA.half.CMPA;
				}

				// At the end of soft-start, turn-on high-side coolmos
				if((Latch_counter == 11) && (sysFlag.EnableSync == 1))
				{
					//set HS coolmos HIGH
					GpioDataRegs.GPASET.bit.GPIO7 = 1;
				}
				// Until Latch time over and Vrect is lower than threshold voltage
				if( (Latch_counter > 295) && (Vrect < Vth))
				{
					//set HS coolmos LOW
					GpioDataRegs.GPACLEAR.bit.GPIO7 = 1;
				}
			}

			// If in positive half, update cmpr in another way,
			else
			{
				//soft-start for the first 12 cycles
				if(Latch_counter < 10)
				{
					// LS GaN duty increases gradually
					// counter multiply by 80
					EPwm2Regs.CMPA.half.CMPA = 750 - ((Latch_counter << 4) + (Latch_counter << 6));

					// HS GaN keeps 0% duty
					EPwm1Regs.CMPA.half.CMPA = 0;
				}
				else
				{
					//LS GaN is the active switch, duty = Duty_out
					EPwm2Regs.CMPA.half.CMPA = 750 - Duty_out;

					//HS GaN is SR switch, duty = 1 - Duty_out
					EPwm1Regs.CMPA.half.CMPA = EPwm2Regs.CMPA.half.CMPA;
				}

				// At the end of soft-start, turn-on low-side coolmos
				if((Latch_counter == 11) && (sysFlag.EnableSync == 1))
				{
					// Set LS coolmos HIGH
					GpioDataRegs.GPASET.bit.GPIO6 = 1;
				}

				// Until Latch time over and Vrect is lower than threshold voltage
				if( (Latch_counter > 295) && (Vrect < Vth))
				{
					// Set LS coolmos LOW
					GpioDataRegs.GPACLEAR.bit.GPIO6 = 1;
				}

			}  // end of if/else: PWM distribution according to polarity

			// Execute voltage loop at 25KHz.
			if(Int1Count == 1)
			{
				// outer voltage loop
				Uerr = Udcref - Udc;
				//limit the error to +-50V  50*4.6534=232
				UpDownLimit16(Uerr, 232, -232);

				//  Voltage PI loop
				UregPout = Uerr * KpV;

				UregIout_pre = UregIout + Uerr * KiV;
				UregPIout_pre = UregIout_pre + UregPout;

				if(UregPIout_pre > VregPILimitUpper)
				{
					if(Uerr > 0)
					{
						Uerr = 0;
					}
				}
				if(UregPIout_pre < VregPILimitLower)
				{
					if(Uerr < 0)
					{
						Uerr = 0;
					}
				}

				// Do real integration after Uerr clamping
				UregIout = UregIout + Uerr * KiV;

				UregPIout1 = UregIout + UregPout;

				// Copy of voltage regulator output before limit action
				UregPIout = UregPIout1;

				//VregPILowerLimit is related to Burst Mode operation
				UpDownLimitfloat(UregPIout, VregPILimitUpper, VregPILimitLower);

				// In burst mode, current reference output is fixed,
				// Make sure it is very high current during burst
				if(sysFlag.BurstMode == 1)
				{
					// In burst mode, deliver fixed current/power to the capacitor
					UregPIout = VregPILimitBurst;
				}

				// the current scaling Kcurrent = 174.08
				// divide PI output by Vrms
				tempf1 = UregPIout * inv_Vrms;
				// divide by Vrms again
				UPI_Iref_out = tempf1 * inv_Vrms;
			}

			Ierr_1 = Ierr;
			Duty_1 = Duty;

			Int1Count ++;
			if(Int1Count >= 4)
			{
				Int1Count = 0;
			}

			//Over current protection, check the current cycle by cycle
			//Ilimit = 1501/68.238 = 22.5A (R19=5.49kOhm)
			if(Irect > 1535) //1535, 1300 for 19A
			{
				//stop everything when an over current occurs  22.5A
				sysFlag.Fault = 1;
			}

			// Current protection according to polarity,
			// Make sure the boost inductor is boosted in the right direction
			// in negative half cycle
			if(sysFlag.Pol == 0)
			{
				// IL_zero + 171 = 2048+171 =  2219; 171/68.238 = 2.5A
				if(IL > 2219)
				{
					sysFlag.Fault = 1;
				}
			}
			// in positive half
			else
			{
				// IL_zero - 171 = 2048-171 = 1877
				if(IL < 1877)
				{
					sysFlag.Fault = 1;
				}
			}
		} // end of the if(!shutdown) clause
	} // end of if(latch_counter!=0) clause

	// if the Latch_counter is 0. do not do all the control, change the mode accordingly
	else
	{
		// in positive half
		if(sysFlag.Pol == 1)
		{
			EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_PRD; 		// INT on counter-zero
			EPwm1Regs.ETPS.bit.INTPRD  = ET_1ST;        	// Generate pulse on 1st event

			EPwm1Regs.ETSEL.bit.SOCASEL	= ET_CTR_PRD;		// Select SOC on counter-zero event
			EPwm1Regs.ETPS.bit.SOCAPRD 	= ET_1ST;			// Generate pulse on 1st event
		}

		// in negative half
		else
		{
			EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO; 		// INT on counter-zero
			EPwm1Regs.ETPS.bit.INTPRD  = ET_1ST;        	// Generate pulse on 1st event

			EPwm1Regs.ETSEL.bit.SOCASEL	= ET_CTR_ZERO;		// Select SOC on counter-zero event
			EPwm1Regs.ETPS.bit.SOCAPRD 	= ET_1ST;			// Generate pulse on 1st event
		}
	}

	sysFlag.Pol_last = sysFlag.Pol;

	Latch_counter ++;


	// Clear INT flag for this timer
	EPwm1Regs.ETCLR.bit.INT = 1;

	// Acknowledge this interrupt to receive more interrupts from group 3
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

interrupt void epwm3_isr(void)
{
	EINT;
	EPwm3TimerIntCount++;

	if(EPwm3TimerIntCount == 2500)
	{
		EPwm3TimerIntCount = 1;
		GpioDataRegs.GPATOGGLE.bit.GPIO31 = 1;
	}

	// Record how many points are collected during a half-cycle.
	MainPoint ++;
	// Calculate the RMS value of input voltage.
	VSqrtSum += (float)Vrect * Vrect;

	// if last slow cycle there is no latch and this cycle there is a effective latch signal, a zero-crossing happens
	// when there is a zero-crossing, the beginning of half-cycle
	if( (sysFlag.Latch == 0) && (sysFlag.Latch_last == 1) )
	{
		tempf1 = (VSqrtSum/MainPoint);
		Vrms = (int) (sqrt(tempf1));

		// Digital filter on Vrms, which sampling frequency is 120Hz.
		Vrms_f += ((Vrms - Vrms_f) >> 2);

		inv_Vrms =1.0/Vrms_f;

		// Calculate the frequency
		// Reset the MainPoint every half cycle
		MainPoint = 0;

		// Reset the sum of Vin square
		VSqrtSum = 0;

		// the whole soft-start code only run when softstart is not over
		if(sysFlag.SoftStartOver == 0)
		{
			// start countering the sscounter when HalfCharged bit is set
			if(sysFlag.HalfCharged == 1)
			{
				// increase the sscounter by 1 every half cycle.
				sscounter ++;
			}

			// when delay is over, bypass NTC at zero-crossing.
			// delay is 100 half-cycle
			if(sscounter > 100)
			{
				//set output high, close the Relay, bypass NTC
				GpioDataRegs.GPASET.bit.GPIO19 = 1;
			}

			// close the loop when it is the positive half
			if((sscounter > 110) && (sysFlag.Pol == 1))
			{
				// before turn-on control loop, set the DC bus reference properly 1.5*Vrms_f
				tempu1 = Vrms_f + (Vrms_f >> 1);
				// for low line, start with 220V  *4.653=1024
				Udcref = (tempu1 > 1002)? tempu1 : 1002;
				Udcref1 = Udcref;
				sysFlag.Stop = 0;   // start the control loop.
			}

			// wait for a few cycle then enable sync-rec
			if((sscounter > 126) && (sysFlag.Pol == 1))
			{
				// stop counting
				sscounter = 127;
				// update the Udcref before filtering, then the Udcref will approach 1805 in a ramp. 388*4.6534=1805
				Udcref1 = 1805;
				// enable sync-rec of the MOSFET
				sysFlag.EnableSync = 1;
				// set SoftStartOver bit to 1.
				sysFlag.SoftStartOver = 1;
			}

			// AC input is over 84V, and DC bus reaches 100V, DC bus cap is half charged
			// 84 * 4.6534 = 391;
			if((Vrms_f > 390) && (Udc > 466))
			{
				// Set the HalfCharged bit, start delay counting
				sysFlag.HalfCharged = 1;
			}
		}

		// Burst mode operation only happens when soft-start is over or when control is enabled
		if(sscounter > 112)
		{
			// code to handle burst mode operation
			// check for burst operation if the operation is already in burst mode
			if(sysFlag.BurstMode == 1)
			{
				// if BurstTurnOff is 1, initiate burst operation,
				// turn-off all gate momentarily for at least half-cycle
				if(Udc > (Udcref + 22))
				{
					// stop all gate, from the beginning of a half-cycle
					sysFlag.BurstTurnOff = 1;
				}
				// if BurstTurnOff is 0, resume normal operation only at the beginning of positive half
				if((Udc < (Udcref - 45)) && (sysFlag.Pol == 1))
				{
					// resume operation, from the beginning of a half-cycle
					sysFlag.BurstTurnOff = 0;
				}
			}
		}

		// make sure when we turn-off BurstMode, the BurstTurnOff bit is set to 0 (Gates all ON).
		if((sysFlag.BurstMode == 0) && (sysFlag.Pol == 1))
		{
			sysFlag.BurstTurnOff = 0;
		}

		if(sysFlag.Fault == 1)
		{
			// Light up the fault LED
			GpioDataRegs.GPBCLEAR.bit.GPIO34 = 1;
			// Open relay to engage the thermistor under fault
			GpioDataRegs.GPACLEAR.bit.GPIO19 = 1;
		}

	}   // end of the code done at zero-crossing
	
	if(Udc <= 0)
	{
		Udc = 1;
	}

	// Calculate inverse of Udc
	inv_Udc = 750.0/Udc;

	// in case Mainpoint get out of control
	//66Hz, 10KHz sampling has 10K/(2*66)=76 points
	//47Hz 10K/(47*2)=107 point
	if(MainPoint > 120)
	{
		MainPoint = 0;
		VSqrtSum = 0;
		Vrms = 10;
		// reset all status bit, and re-initialize important variables
		// at the beginning stop the PWM.
		sysFlag.Stop = 1;
		sysFlag.EnableSync = 0;
		// initial value 0, this bit is changed to 1 when DC bus reached an level
		sysFlag.HalfCharged = 0;
		// initial value 0, no burst turn-off
		sysFlag.BurstTurnOff = 0;
		// initial value 0, no burst mode, do not turn off gate momentarily
		sysFlag.BurstMode = 0;
		// initial value 0, converter has not completed soft-start sequence
		sysFlag.SoftStartOver = 0;
		// no light load
		sysFlag.LightLoad = 0;
		sysFlag.LightLoad_last = 0;
		sscounter = 0;
		// reset light load condition counter
		lightload = 0;

		// reset all controller variables
		Uerr = 0;
		Uerr_last = 0;
		Icntr = 0;
		Ierr = 0;
		Ierr_1 = 0;
		Duty = 10;
		Duty_1 = 10;

		//initially will give out a very small current
		UregIout = 35752.0;

		// re-open the relay, prepare for the next power-on soft-start sequence
		// Do not try to open the relay unless DC bus is already discharged.
		if(Udc < 45)
		{
			//set output low, open the Relay, engage NTC
			GpioDataRegs.GPACLEAR.bit.GPIO19 = 1;
		}
	}

	// save latch signal for the use of next cycle.
	sysFlag.Latch_last = sysFlag.Latch;

	//update the limit for voltage loops
	// 23A in DSP 23*68.238=1570
	tempf3 = 1570.0 * Vrms_f;
	VregILimit = tempf3;

	// 15A in DSP 15*68.238=1024
	tempf3 = 1024.0 * Vrms_f;
	VregPILimitUpper = tempf3;

	// 0.25A in DSP   0.25*68.238= 17.0595
	tempf3 = 17.0 * Vrms_f;
	VregPILimitLower = tempf3;

	// lower limit for the voltage loops, this is related to the burst mode operation.
	// Below this current level, burst mode will happens
	// 0.5A in DSP   0.5*68.238= 34.119
	tempf3 = 34.119 * Vrms_f;
	VregPILimitBurst = tempf3;

	if(Udcref < Udcref1)
	{
		//slowly ramp the Vdcref up to rated DC bus reference
		if(EPwm3TimerIntCount == 1000)
		{
			Udcref = Udcref + 20;
		}
		Udcref = ( Udcref >= Udcref1)? Udcref1 : Udcref;
	}

	// check lightload condition and burstmode only when soft-start is over,
	// check this when burst mode is off
	if((sscounter > 112) && (sysFlag.BurstMode == 0))
	{
		// check if the controller output is too low
		if(UregPIout1 < VregPILimitBurst)
		{
			// set the LighLoad flag for this control cycle
			sysFlag.LightLoad = 1;
		}
		else
		{
			// otherwise, LightLoad flag is reset
			sysFlag.LightLoad = 0;
			// lightload counter is reset back to 0 immediately
			lightload = 0;
		}

		//  if it is LightLoad this cycle and also last cycle
		if((sysFlag.LightLoad == 1) && (sysFlag.LightLoad_last == 1))
		{
			// increase lightload counter only if hit the light load condition continuously
			lightload ++;
		}
		// if the reference hit lightload conidtion for sometime 10ms, set to burstmode
		if(lightload > 100)
		{
			sysFlag.BurstMode = 1;
			// limit light-load counter
			lightload = 101;
		}
	}

	// save LightLoad flag for use in the next cycle
	sysFlag.LightLoad_last = sysFlag.LightLoad;

	// check to exit Burst Mode when the Vout can not be sustain under Burst Operation
	// if in Burst Mode and voltage drops too much, then needs to exit burst mode
	if((sysFlag.SoftStartOver == 1) && (sysFlag.BurstMode == 1))
	{
		// if light load continues, Udc will be very well regulated
		if(Udc < (Udcref - 90))
		{
			sysFlag.BurstMode = 0;
			lightload = 0;
			//initially will give out a very small current when exit burst mode
			UregIout = 35752.0;
		}
	}

	// parameters for high-line operation, initial value is good for low line
	// if Vrms bigger than 190, use parameter for 220V
	// 190V*4.6534 = 885
	if((Vrms_f > 885) && (VLine_flag == 0))
	{
		VLine_flag = 1; 			// set high line flag
		K1I = 0.515186;             //   230Vin
		K2I = 0.454909;       		//   230Vin
		//threshold for high line changed to 35; for low line, it is default 20.
		Vth = 35;
		// threshold for control, for high line changed to 20, low line is 10 by default
		Vth2 = 20;
	}
	if((Vrms_f < 815) && (VLine_flag == 1)) //175V
	{
		VLine_flag = 0;				// set low line flag
		K1I = 0.347452;             //   115Vin
		K2I = 0.326258;       		//   115Vin
		//threshold for high line changed to 35; for low line, it is default 20.
		Vth = 20;
		// threshold for control, for high line changed to 20, low line is 10 by default
		Vth2 = 10;
	}

	//DC bus over voltage protection
	//437*4.6534=2033
	if(Udc > 2033)
	{
		//stop everything when an overvoltage occurs
		sysFlag.Fault = 1;
	}

	// Clear INT flag for this timer
	EPwm3Regs.ETCLR.bit.INT = 1;

	// Acknowledge this interrupt to receive more interrupts from group 3
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}


void InitEPwm1Gpio(void)
{
	EALLOW;
	GpioCtrlRegs.GPAPUD.bit.GPIO0 = 1;    // Disable pull-up on GPIO0 (EPWM1A)
	GpioCtrlRegs.GPAPUD.bit.GPIO1 = 1;    // Disable pull-up on GPIO1 (EPWM1B)
	GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 1;   // Configure GPIO0 as EPWM1A
	GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 1;   // Configure GPIO1 as EPWM1B
	EDIS;
}


void InitEPwm2Gpio(void)
{
	EALLOW;
	GpioCtrlRegs.GPAPUD.bit.GPIO2 = 1;    // Disable pull-up on GPIO2 (EPWM2A)
	GpioCtrlRegs.GPAPUD.bit.GPIO3 = 1;    // Disable pull-up on GPIO3 (EPWM2B)
	GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 1;   // Configure GPIO2 as EPWM2A
	GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 1;   // Configure GPIO3 as EPWM2B
	EDIS;
}

void InitEPwm3Gpio(void)
{
	EALLOW;
	GpioCtrlRegs.GPAPUD.bit.GPIO4 = 1;    // Disable pull-up on GPIO4 (EPWM3A)
	GpioCtrlRegs.GPAPUD.bit.GPIO5 = 1;    // Disable pull-up on GPIO5 (EPWM3B)
	GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 1;   // Configure GPIO4 as EPWM3A
	GpioCtrlRegs.GPAMUX1.bit.GPIO5 = 1;   // Configure GPIO5 as EPWM3B
	EDIS;
}


void InitEPwm1Example()
{
	EPwm1Regs.TBPRD = 750;                        			// Set timer period  150M/1500=100KHz
	EPwm1Regs.TBPHS.half.TBPHS = 0x0000;           			// Phase is 0
	EPwm1Regs.TBCTR = 0x0000;                      			// Clear counter

	// Setup TBCLK
	EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; 			// Count up
	EPwm1Regs.TBCTL.bit.PHSEN = TB_DISABLE;        			// Disable phase loading
	EPwm1Regs.TBCTL.bit.PRDLD = TB_SHADOW;
	EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;    			//Sync down-stream module

	EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       			// Clock ratio to SYSCLKOUT
	EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;

	EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;    			// Load registers immediately
	EPwm1Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	EPwm1Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO_PRD;     	// no effect in immediately mode
	EPwm1Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO_PRD;

	// Setup compare
	EPwm1Regs.CMPA.half.CMPA = 375;

	// Set actions
	EPwm1Regs.AQCTLA.bit.CAU = AQ_CLEAR;             // Set PWM1A on Zero
	EPwm1Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm1Regs.AQCTLB.bit.CAU = AQ_CLEAR;          	// Set PWM1A on Zero
	EPwm1Regs.AQCTLB.bit.CAD = AQ_SET;

	//Trip event setup
	EALLOW;
	EPwm1Regs.TZSEL.bit.OSHT1 = 1;  		// enable one-shot trip for PWM1
	EPwm1Regs.TZCTL.bit.TZA = TZ_FORCE_LO;  // set EPWM1A to low at default
	EPwm1Regs.TZFRC.bit.OST = 1; 			// shutdown EPWM1 at beginning
	EDIS;

	// Interrupt where we have the control
	EPwm1Regs.ETSEL.bit.INTEN = 1;              // Enable INT
	EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO; 	// INT on counter-zero
	EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;        	// Generate pulse on 1st event

	EPwm1Regs.ETSEL.bit.SOCAEN	= 1;			// Enable SOC on A group
	EPwm1Regs.ETSEL.bit.SOCASEL	= ET_CTR_ZERO;	// Select SOC on counter-zero event
	EPwm1Regs.ETPS.bit.SOCAPRD 	= ET_1ST;		// Generate pulse on 1st event

}


void InitEPwm2Example()
{

	EPwm2Regs.TBPRD = 750;                        // Set timer period
	EPwm2Regs.TBPHS.half.TBPHS = 0x0000;          // Phase is 0
	EPwm2Regs.TBCTR = 0x0000;                     // Clear counter

	// Setup TBCLK
	EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;  // Count up
	EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;        	// Enable phase loading
	EPwm2Regs.TBCTL.bit.PRDLD = TB_SHADOW;

	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; 		//sync flow-through

	EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;    // Load registers immediately
	EPwm2Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	EPwm2Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO_PRD;     // no effect in immediately mode
	EPwm2Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO_PRD;

	EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       // Clock ratio to SYSCLKOUT
	EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;          // Slow just to observe on the scope

	// Setup compare
	EPwm2Regs.CMPA.half.CMPA = 375;

	// Set actions
	EPwm2Regs.AQCTLA.bit.CAU = AQ_SET;             // Set PWM2A on Zero
	EPwm2Regs.AQCTLA.bit.CAD = AQ_CLEAR;


	EPwm2Regs.AQCTLB.bit.CAU = AQ_SET;           // Set PWM2A on Zero
	EPwm2Regs.AQCTLB.bit.CAD = AQ_CLEAR;

	//Trip event set up
	EALLOW;
	EPwm2Regs.TZSEL.bit.OSHT1 = 1;  			//enable one-shot trip for PWM1
	EPwm2Regs.TZCTL.bit.TZA = TZ_FORCE_LO;   	// set EPWM2A to high at default
	EPwm2Regs.TZFRC.bit.OST = 1; 				// shutdown EPWM2 at the beginning
	EDIS;

}

void InitEPwm3Example()
{

	EPwm3Regs.TBPRD = 7500;                         // Set timer period
	EPwm3Regs.TBPHS.half.TBPHS = 0x0000;            // Phase is 0
	EPwm3Regs.TBCTR = 0x0000;                       // Clear counter


	// Setup TBCLK
	EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; // Count up
	EPwm3Regs.TBCTL.bit.PHSEN = TB_DISABLE;        // Disable phase loading
	EPwm3Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       // Clock ratio to SYSCLKOUT
	EPwm3Regs.TBCTL.bit.CLKDIV = TB_DIV1;          // Slow so we can observe on the scope
	EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;

	// slow Interrupt at 10KHz
	EPwm3Regs.ETSEL.bit.INTEN = 1;
	EPwm3Regs.ETSEL.bit.INTSEL = ET_CTR_PRD;   		// Select Int from counter = PRD
	EPwm3Regs.ETPS.bit.INTPRD = ET_1ST;        		// Generate pulse on 1st event

	EPwm3Regs.AQSFRC.bit.RLDCSF = 0x3;      		// load immediately

}


void InitAdc(void)
{
    extern void DSP28x_usDelay(Uint32 Count);


    // *IMPORTANT*
	// The ADC_cal function, which  copies the ADC calibration values from TI reserved
	// OTP into the ADCREFSEL and ADCOFFTRIM registers, occurs automatically in the
	// Boot ROM. If the boot ROM code is bypassed during the debug process, the
	// following function MUST be called for the ADC to function according
	// to specification. The clocks to the ADC MUST be enabled before calling this
	// function.
	// See the device data manual and/or the ADC Reference
	// Manual for more information.

	    EALLOW;
		SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1;
		ADC_cal();
		EDIS;




    // To powerup the ADC the ADCENCLK bit should be set first to enable
    // clocks, followed by powering up the bandgap, reference circuitry, and ADC core.
    // Before the first conversion is performed a 5ms delay must be observed
	// after power up to give all analog circuits time to power up and settle

    // Please note that for the delay function below to operate correctly the
	// CPU_RATE define statement in the DSP2833x_Examples.h file must
	// contain the correct CPU clock period in nanoseconds.

    AdcRegs.ADCTRL3.all = 0x00E0;  // Power up bandgap/reference/ADC circuits
    DELAY_US(5000L);         // Delay before converting ADC channels
}


void ConfigAdc1()  // configure ADC, first sample the Vref
{
	EALLOW;

   	AdcRegs.ADCTRL3.bit.ADCCLKPS = 0x0;			//ADC module clock = HSPCLK/1= 25MHz
   	AdcRegs.ADCTRL1.bit.CPS = 0;
	AdcRegs.ADCTRL1.bit.ACQ_PS = 0x1;    // acqusition time 1+1 ADCLK period.
   	AdcRegs.ADCTRL1.bit.SEQ_CASC = 1;     // cascaded mode

	AdcRegs.ADCMAXCONV.all = 0x0003;       // Setup 4 adc conversion

	AdcRegs.ADCCHSELSEQ1.bit.CONV00 = 0x2; // Setup ADCINA2 as 1st SEQ1 conv.
    AdcRegs.ADCCHSELSEQ1.bit.CONV01 = 0x2; // Setup ADCINA2 as 2nd SEQ1 conv.
	AdcRegs.ADCCHSELSEQ1.bit.CONV02 = 0x2; // Setup ADCINA2 as 3rd SEQ1 conv.
    AdcRegs.ADCCHSELSEQ1.bit.CONV03 = 0x2; // Setup ADCINA2 as 4th SEQ1 conv.


	EDIS;
}

void ConfigAdc2()  // configure ADC to sample current and voltage, for the control purpose
{
	EALLOW;

	AdcRegs.ADCMAXCONV.all = 0x0003;       // Setup 5 adc conversion

    AdcRegs.ADCTRL2.bit.EPWM_SOCA_SEQ1 = 1;// Enable SOCA from ePWM to start SEQ1

	AdcRegs.ADCCHSELSEQ1.bit.CONV00 = 0x2; // Setup ADCINA2 as 1st SEQ1 conv.
    AdcRegs.ADCCHSELSEQ1.bit.CONV01 = 0x3; // Setup ADCINA3 as 2nd SEQ1 conv.
	AdcRegs.ADCCHSELSEQ1.bit.CONV02 = 0x0; // Setup ADCINA0 as 3rd SEQ1 conv.
    AdcRegs.ADCCHSELSEQ1.bit.CONV03 = 0x1; // Setup ADCINA1 as 4th SEQ1 conv.


	EDIS;
}


void InitGPIOs(void)
{
	// Enable an GPIO output on GPIO7, the direction is input. (Input the polarity)
	// Enable an GPIO output on GPIO18, the direction is output. (Fault), set it to 1 high.
	EALLOW;

	GpioCtrlRegs.GPAPUD.bit.GPIO6 = 0;   // Enable pullup on GPIO6
	GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 0;  // GPIO6 = GPIO6
	GpioCtrlRegs.GPADIR.bit.GPIO6 = 1;   //GPIO as output
	GpioDataRegs.GPACLEAR.bit.GPIO6 = 1;   //set output LOW

	GpioCtrlRegs.GPAPUD.bit.GPIO7 = 0;   // Enable pullup on GPIO7
	GpioCtrlRegs.GPAMUX1.bit.GPIO7 = 0;  // GPIO7 = GPIO7
	GpioCtrlRegs.GPADIR.bit.GPIO7 = 1;   //GPIO as output
	GpioDataRegs.GPACLEAR.bit.GPIO7 = 1;   //set output LOW

	GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;   // Enable pullup on GPIO17
	GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 0;  // GPIO17 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO17 = 0;   // GPIO17 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO18 = 0;   // Enable pullup on GPIO18
	GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 0;  // GPIO18 = GPIO18
	GpioCtrlRegs.GPADIR.bit.GPIO18 = 1;   //GPIO as output
	GpioDataRegs.GPASET.bit.GPIO18 = 1;   //set output HIGH

	GpioCtrlRegs.GPAPUD.bit.GPIO19 = 0;   // Enable pullup on GPIO18
	GpioCtrlRegs.GPAMUX2.bit.GPIO19 = 0;  // GPIO18 = GPIO18
	GpioCtrlRegs.GPADIR.bit.GPIO19 = 1;   //GPIO as output
	GpioDataRegs.GPACLEAR.bit.GPIO19 = 1;   //set output LOW

	GpioCtrlRegs.GPAPUD.bit.GPIO31 = 0;   // Enable pullup on GPIO12
	GpioCtrlRegs.GPAMUX2.bit.GPIO31 = 0;  // GPIO12 = GPIO12
	GpioCtrlRegs.GPADIR.bit.GPIO31 = 1;   //GPIO as output
	GpioDataRegs.GPACLEAR.bit.GPIO31 = 1;   //set output HIGH

	GpioCtrlRegs.GPBPUD.bit.GPIO34 = 0;   // Enable pullup on GPIO12
	GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;  // GPIO12 = GPIO12
	GpioCtrlRegs.GPBDIR.bit.GPIO34 = 1;   //GPIO as output
	GpioDataRegs.GPBSET.bit.GPIO34 = 1;   //set output HIGH

	EDIS;
}

// end
